==> visual-third-party-3.5-sources.jar: com/tenqube/visual_third/VisualServiceImpl.java <==
package com.tenqube.visual_third;

import android.Manifest;
import android.content.pm.PackageManager;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;

import com.tenqube.visual_third.api.VisualApi;
import com.tenqube.visual_third.exception.AuthException;
import com.tenqube.visual_third.exception.ParameterException;
import com.tenqube.visual_third.manager.PrefManager;
import com.tenqube.visual_third.model.api.SignUpRequest;
import com.tenqube.visual_third.model.api.SignUpResponse;
import com.tenqube.visual_third.repository.VisualRepository;
import com.tenqube.visual_third.ui.OnResultListener;
import com.tenqube.visual_third.ui.VisualWebActivity;

import java.util.Calendar;

import tenqube.parser.core.ParserService;

import static com.tenqube.visual_third.Constants.DEV;
import static com.tenqube.visual_third.Constants.PROD;
import static com.tenqube.visual_third.manager.PrefManager.UID;
import static com.tenqube.visual_third.util.Validator.notNull;
import static tenqube.parser.core.ParserService.mIsDebug;
import static tenqube.parser.util.LogUtil.LOGI;

public class VisualServiceImpl implements VisualService {

    public static final String TAG = VisualServiceImpl.class.getSimpleName();

    private PrefManager prefManager;
    private AppCompatActivity activity;
    private VisualApi api;

    private ParserService parserService;

    private boolean isDoingSignup;
    private boolean isDoingStartVisual;

    public VisualServiceImpl(@NonNull AppCompatActivity activity, String apiKey, String qualifier) throws ParameterException {

        if(TextUtils.isEmpty(apiKey)) throw new ParameterException("apiKey is null");

        if(TextUtils.isEmpty(qualifier) || !(Constants.DEV.equals(qualifier) || Constants.PROD.equals(qualifier)))
            throw new ParameterException("check qualifier value (" + DEV + ", " + PROD + ")");

        this.activity = activity;

        VisualRepository.getInstance(activity);
        prefManager = PrefManager.getInstance(activity);
        prefManager.saveStringValue(PrefManager.API_KEY, apiKey);
        prefManager.saveStringValue(PrefManager.QUALIFIER, qualifier);
        api = VisualApi.getInstance(activity);
        parserService = ParserService.getInstance(activity);
    }

    @Override
    public void signUp(@NonNull final String custId, @NonNull final OnResultListener onResultListener) throws ParameterException {
        LOGI(TAG, "가입", mIsDebug);
        if(isDoingSignup) return;
        isDoingSignup = true;
        final SignUpRequest signUpRequest = new SignUpRequest(custId);
        notNull(onResultListener);

        if(!TextUtils.isEmpty(prefManager.loadStringValue(UID, ""))) {
            onResultListener.onResult(Constants.SignupResult.ALREADY_JOINED, "already joined");
            isDoingSignup = false;
            return;
        }

        new Thread(new Runnable() {
            @Override
            public void run() {
                SignUpResponse response = api.signUp(signUpRequest);

                if (response == null) {
                    onResultListener.onResult(Constants.SignupResult.FAIL, "network error");
                } else {

                    // login request succeed, new token generated
                    prefManager.saveStringValue(PrefManager.SEARCH_URL, response.getResults().getSearch().getUrl());
                    prefManager.saveStringValue(PrefManager.SEARCH_API_KEY, response.getResults().getSearch().getApiKey());
                    prefManager.saveStringValue(UID, response.getResults().getAuthorization().getSdk());
                    prefManager.saveLongValue(PrefManager.SIGN_UP_TIME, Calendar.getInstance().getTimeInMillis());
                    if (response.getResults().getWeb() != null)
                        prefManager.saveStringValue(PrefManager.WEB_URL, response.getResults().getWeb().getUrl());

                    onResultListener.onResult(Constants.SignupResult.SUCCESS, "success");
                }

                isDoingSignup = false;
            }
        }).start();
    }

    @Override
    public void startVisual(@NonNull String custId, final @NonNull OnResultListener resultListener) throws ParameterException, SecurityException {
        LOGI(TAG, "웹 가계부 호출", mIsDebug);
        if(isDoingStartVisual) return;

        isDoingStartVisual = true;

        notNull(resultListener);

        checkSMSPermission();

        signUp(custId, new OnResultListener() {
            @Override
            public void onResult(int signUpResult, String msg) {

                switch (signUpResult) {
                    case Constants.SignupResult.ALREADY_JOINED:
                    case Constants.SignupResult.SUCCESS:
                        try {
                            VisualWebActivity.startActivityForResult(activity, "");
                        } catch (AuthException e) {
                            resultListener.onResult(Constants.SignupResult.FAIL, e.toString());
                            return;
                        } finally {
                            isDoingStartVisual = false;
                        }
                        break;
                }

                resultListener.onResult(signUpResult, msg);
                isDoingStartVisual = false;
            }
        });
    }

    private void checkSMSPermission() throws SecurityException {
        if (!prefManager.isEnabled(PrefManager.BULK_EXECUTED, false) &&
                ActivityCompat.checkSelfPermission(activity, Manifest.permission.READ_SMS) != PackageManager.PERMISSION_GRANTED &&
                ActivityCompat.checkSelfPermission(activity, Manifest.permission.RECEIVE_SMS) != PackageManager.PERMISSION_GRANTED &&
                ActivityCompat.checkSelfPermission(activity, Manifest.permission.RECEIVE_MMS) != PackageManager.PERMISSION_GRANTED) {

            throw new SecurityException("Please grant permissions " +
                    "(Manifest.permission.READ_SMS, " +
                    "Manifest.permission.RECEIVE_SMS, " +
                    "Manifest.permission.RECEIVE_MMS)");
        }
    }

    @Override
    public void setDebugMode(boolean isDebug) {
        LOGI(TAG, "setDebugMode", mIsDebug);

        if(isActive() && parserService != null) {
            parserService.setDebugMode(isDebug);
        }
    }

    @Override
    public void initSDK() {
        LOGI(TAG, "initSDK", mIsDebug);

        if(isActive() && parserService != null) {
            parserService.initDb();
        }

    }

    private boolean isActive() {
        return activity != null && !activity.isFinishing();
    }
}

==> visual-third-party-4.0-sources.jar: com/tenqube/visual_third/VisualServiceImpl.java <==
package com.tenqube.visual_third;

import android.Manifest;
import android.content.pm.PackageManager;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;

import com.tenqube.visual_third.api.VisualApi;
import com.tenqube.visual_third.exception.AuthException;
import com.tenqube.visual_third.exception.ParameterException;
import com.tenqube.visual_third.manager.PrefManager;
import com.tenqube.visual_third.model.api.SignUpRequest;
import com.tenqube.visual_third.model.api.SignUpResponse;
import com.tenqube.visual_third.repository.VisualRepository;
import com.tenqube.visual_third.ui.OnResultListener;
import com.tenqube.visual_third.ui.VisualWebActivity;

import java.util.Calendar;

import tenqube.parser.core.ParserService;

import static com.tenqube.visual_third.Constants.DEV;
import static com.tenqube.visual_third.Constants.PROD;
import static com.tenqube.visual_third.manager.PrefManager.UID;
import static com.tenqube.visual_third.util.Validator.notNull;
import static tenqube.parser.core.ParserService.mIsDebug;
import static tenqube.parser.util.LogUtil.LOGI;

public class VisualServiceImpl implements VisualService {

    public static final String TAG = VisualServiceImpl.class.getSimpleName();

    private PrefManager prefManager;
    private AppCompatActivity activity;
    private VisualApi api;

    private ParserService parserService;

    private boolean isDoingSignup;
    private boolean isDoingStartVisual;

    public VisualServiceImpl(@NonNull AppCompatActivity activity, String apiKey, String qualifier) throws ParameterException {

        if(TextUtils.isEmpty(apiKey)) throw new ParameterException("apiKey is null");

        if(TextUtils.isEmpty(qualifier) || !(Constants.DEV.equals(qualifier) || Constants.PROD.equals(qualifier)))
            throw new ParameterException("check qualifier value (" + DEV + ", " + PROD + ")");

        this.activity = activity;

        VisualRepository.getInstance(activity);
        prefManager = PrefManager.getInstance(activity);
        prefManager.saveStringValue(PrefManager.API_KEY, apiKey);
        prefManager.saveStringValue(PrefManager.QUALIFIER, qualifier);
        api = VisualApi.getInstance(activity);
        parserService = ParserService.getInstance(activity);
    }

    @Override
    public void signUp(@NonNull final String custId, @NonNull final OnResultListener onResultListener) throws ParameterException {
        LOGI(TAG, "가입", mIsDebug);
        if(isDoingSignup) return;
        isDoingSignup = true;
        final SignUpRequest signUpRequest = new SignUpRequest(custId);
        notNull(onResultListener);

        if(!TextUtils.isEmpty(prefManager.loadStringValue(UID, ""))) {
            onResultListener.onResult(Constants.SignupResult.ALREADY_JOINED, "already joined");
            isDoingSignup = false;
            return;
        }

        new Thread(new Runnable() {
            @Override
            public void run() {
                SignUpResponse response = api.signUp(signUpRequest);

                if (response == null) {
                    onResultListener.onResult(Constants.SignupResult.FAIL, "network error");
                } else {

                    // login request succeed, new token generated
                    prefManager.saveStringValue(PrefManager.SEARCH_URL, response.getResults().getSearch().getUrl());
                    prefManager.saveStringValue(PrefManager.SEARCH_API_KEY, response.getResults().getSearch().getApiKey());
                    prefManager.saveStringValue(UID, response.getResults().getAuthorization().getSdk());
                    prefManager.saveLongValue(PrefManager.SIGN_UP_TIME, Calendar.getInstance().getTimeInMillis());
                    if (response.getResults().getWeb() != null)
                        prefManager.saveStringValue(PrefManager.WEB_URL, response.getResults().getWeb().getUrl());

                    onResultListener.onResult(Constants.SignupResult.SUCCESS, "success");
                }

                isDoingSignup = false;
            }
        }).start();
    }

    @Override
    public void startVisual(@NonNull String custId, final @NonNull OnResultListener resultListener) throws ParameterException, SecurityException {
        LOGI(TAG, "웹 가계부 호출", mIsDebug);
        if(isDoingStartVisual) return;

        isDoingStartVisual = true;

        notNull(resultListener);

        checkSMSPermission();

        signUp(custId, new OnResultListener() {
            @Override
            public void onResult(int signUpResult, String msg) {

                switch (signUpResult) {
                    case Constants.SignupResult.ALREADY_JOINED:
                    case Constants.SignupResult.SUCCESS:
                        try {
                            VisualWebActivity.startActivity(activity, "");
                        } catch (AuthException e) {
                            resultListener.onResult(Constants.SignupResult.FAIL, e.toString());
                            return;
                        } finally {
                            isDoingStartVisual = false;
                        }
                        break;
                }

                resultListener.onResult(signUpResult, msg);
                isDoingStartVisual = false;
            }
        });
    }

    @Override
    public boolean isActiveTranPopup() {
        return prefManager.isEnabled(PrefManager.TRAN_POPUP, true);
    }

    @Override
    public void setTranPopup(boolean isActive) {
        prefManager.saveBoolean(PrefManager.TRAN_POPUP, isActive);
    }

    private void checkSMSPermission() throws SecurityException {
        if (!prefManager.isEnabled(PrefManager.BULK_EXECUTED, false) &&
                ActivityCompat.checkSelfPermission(activity, Manifest.permission.READ_SMS) != PackageManager.PERMISSION_GRANTED &&
                ActivityCompat.checkSelfPermission(activity, Manifest.permission.RECEIVE_SMS) != PackageManager.PERMISSION_GRANTED &&
                ActivityCompat.checkSelfPermission(activity, Manifest.permission.RECEIVE_MMS) != PackageManager.PERMISSION_GRANTED) {

            throw new SecurityException("Please grant permissions " +
                    "(Manifest.permission.READ_SMS, " +
                    "Manifest.permission.RECEIVE_SMS, " +
                    "Manifest.permission.RECEIVE_MMS)");
        }
    }

    @Override
    public void setDebugMode(boolean isDebug) {
        LOGI(TAG, "setDebugMode", mIsDebug);

        if(isActive() && parserService != null) {
            parserService.setDebugMode(isDebug);
        }
    }

    @Override
    public void initSDK() {
        LOGI(TAG, "initSDK", mIsDebug);

        if(isActive() && parserService != null) {
            parserService.initDb();
        }

    }

    private boolean isActive() {
        return activity != null && !activity.isFinishing();
    }
}
